function [uc,vc,pc,wc,divc,ut,vt,pt,wt,xx,yy,TFINAL] =  ...
                                           nse2d_coll(N,nu,cfl,tfinal);
%
%  NOTE: we index from 1:N+1 in MATLAB, not 0:N
%

NP1=N+1; NM1=N-1;
[D1,x]=cheb(N); y=x; Dx = D1; Dy = D1';; D2 = D1*D1; D2T = D2';
[xx,yy]=meshgrid(x,y); xx=xx'; yy=yy'; 

% compute stable dt and number of time steps

dx=x(1)-x(2);  dt=cfl*dx;  nts=floor(tfinal/dt);  TFINAL=nts*dt;

%%% Setup collocation matrix CD for PPE

[Q,evals]=eig(D2(2:N,2:N)); evals=diag(evals); 
QT=Q'; invQ =inv(Q); invQT = invQ';
Qevals_mat = 1-(nu*dt/2)*(repmat(evals,1,NM1)+repmat(evals',NM1,1));

c0_m = -D1(1,NP1); c0_p = D1(NP1,NP1); cN_p = -D1(NP1,1); cN_m = D1(1,1);
b0 = -c0_p*D1(1,2:N)-c0_m*D1(NP1,2:N); bN = -cN_m*D1(NP1,2:N)-cN_p*D1(1,2:N);
e = c0_p*cN_m - c0_m*cN_p;

[R,evals] = eig(D2(2:N,2:N)+(1/e)*(D2(2:N,1)*b0+D2(2:N,NP1)*bN)); 
evals = diag(evals); [evals,ind] = sort(evals); evals = flipud(evals); 
evals(1)=0;  R=R(:,flipud(ind)'); RT = R'; invR =inv(R); invRT = invR';
Revals_mat = repmat(evals,1,NM1)+repmat(evals',NM1,1); Revals_mat(1,1)=1;

[tu,tv,tp,tw,fu1,fu2,fu3,fv1,fv2,fv3, ...
fu1_x,fu2_x,fu3_x,fv1_y,fv2_y,fv3_y] = accfuncsnl(xx,yy,nu);
u2=tu; v2=tv; p2=tp; u1=tf1(-dt)*tu; v1=tf1(-dt)*tv; p1=tf1(-dt)*tp;

% 

A = [D1(1,1) D1(1,NP1) ; D1(NP1,1) D1(NP1,NP1)]; cp=zeros(2,2);
unew=zeros(NP1); vnew=zeros(NP1); pnew=zeros(NP1);
disp(' ')

% TIME STEPPING 

for k=1:nts

    time=(k-1)*dt;  
    if (mod(k,25) == 0) 
       disp(['step : ',num2str(k),'  time : ',num2str(time)]); 
    end
 
%   update velocities first

    f_time=time+dt/2;    
    FU = tf1(f_time)*fu1 + tf2(f_time)*fu2 + tf3(f_time)*fu3;  
    FV = tf1(f_time)*fv1 + tf2(f_time)*fv2 + tf3(f_time)*fv3;
    FU_x = tf1(f_time)*fu1_x + tf2(f_time)*fu2_x + tf3(f_time)*fu3_x;
    FV_y = tf1(f_time)*fv1_y + tf2(f_time)*fv2_y + tf3(f_time)*fv3_y;

    NLT_u1 = u1.*(Dx*u1)+v1.*(u1*Dy); NLT_v1 = u1.*(Dx*v1)+v1.*(v1*Dy);
    NLT_u2 = u2.*(Dx*u2)+v2.*(u2*Dy); NLT_v2 = u2.*(Dx*v2)+v2.*(v2*Dy);    
    lap_u2 = D2*u2+u2*D2T; lap_v2 = D2*v2+v2*D2T;

    unew = u2+dt*(0.5*nu*lap_u2-(3*NLT_u2-NLT_u1)/2-Dx*(3*p2-p1)/2+FU);
    vnew = v2+dt*(0.5*nu*lap_v2-(3*NLT_v2-NLT_v1)/2-(3*p2-p1)/2*Dy+FV);

    unew(2:N,2:N) = Q*((invQ*unew(2:N,2:N)*invQT)./Qevals_mat)*QT;
    vnew(2:N,2:N) = Q*((invQ*vnew(2:N,2:N)*invQT)./Qevals_mat)*QT;
    unew(1,:) = 0; unew(NP1,:) = 0; unew(:,1) = 0; unew(:,NP1) = 0;   
    vnew(1,:) = 0; vnew(NP1,:) = 0; vnew(:,1) = 0; vnew(:,NP1) = 0;   

% pressure

    f_time=time+dt;    
    FU = tf1(f_time)*fu1 + tf2(f_time)*fu2 + tf3(f_time)*fu3;  
    FV = tf1(f_time)*fv1 + tf2(f_time)*fv2 + tf3(f_time)*fv3;
    FU_x = tf1(f_time)*fu1_x + tf2(f_time)*fu2_x + tf3(f_time)*fu3_x;
    FV_y = tf1(f_time)*fv1_y + tf2(f_time)*fv2_y + tf3(f_time)*fv3_y;

    unew_x = Dx*unew; unew_y = unew*Dy; vnew_x = Dx*vnew; vnew_y = vnew*Dy;      
    NLT_u = unew.*unew_x+vnew.*unew_y; NLT_v = unew.*vnew_x+vnew.*vnew_y;

    pnew = 0.5*(unew_x + vnew_y).^2  + FU_x + FV_y   ...
         - (unew_x.^2 + 2*vnew_x.*unew_y + vnew_y.^2);

% compute BC for PPE
    
    w   = vnew_x - unew_y;  w_x = Dx*w;  w_y = w*Dy;     
    px_m = FU(NP1,:) - nu*w_y(NP1,:); px_p = FU(1,:) - nu*w_y(1,:);
    py_m = FV(:,NP1) + nu*w_x(:,NP1); py_p = FV(:,1) + nu*w_x(:,1); 

% compute adjusted RHS accounting for BC

    i=2:N;j=2:N;
    pnew(i,j) = pnew(i,j)-(1/e)*(D2(i,  1)*(c0_m*px_m(j)+c0_p*px_p(j)) + ...
                                 D2(i,NP1)*(cN_m*px_m(j)+cN_p*px_p(j)) + ...
                                 (c0_m*py_m(i)+c0_p*py_p(i))*D2(j,  1)'+ ...
                                 (cN_m*py_m(i)+cN_p*py_p(i))*D2(j,NP1)');

% solve PPE via diagonalized system
 
    pnew(i,j) = invR*pnew(i,j)*invRT; pnew(2,2)=0; 
    pnew(i,j) = R*(pnew(i,j)./Revals_mat)*RT;

    pnew(  1,j)=(b0*pnew(i,j) + c0_m*px_m(j) + c0_p*px_p(j)) / e;
    pnew(NP1,j)=(bN*pnew(i,j) + cN_m*px_m(j) + cN_p*px_p(j)) / e;
    pnew(i,  1)=(pnew(i,j)*b0' + c0_m*py_m(i) + c0_p*py_p(i)) / e; 
    pnew(i,NP1)=(pnew(i,j)*bN' + cN_m*py_m(i) + cN_p*py_p(i)) / e;

% recover corner points

   cp(1,1) = px_p(  1)-D1(  1,j)*pnew(i,  1);
   cp(2,1) = px_m(  1)-D1(NP1,j)*pnew(i,  1);
   cp(1,2) = px_p(NP1)-D1(  1,j)*pnew(i,NP1);
   cp(2,2) = px_m(NP1)-D1(NP1,j)*pnew(i,NP1);
   cp = A\cp;   pnew(1,1)=cp(1,1); pnew(NP1,1)=cp(2,1); 
   pnew(1,NP1)=cp(1,2); pnew(NP1,NP1)=cp(2,2);

%  update variables for time loop

   u1=u2; u2=unew; v1=v2; v2=vnew; p1=p2; p2=pnew;
   
end  % time stepping

% get true solution at TFINAL

tfac=tf1(TFINAL);
ut = tfac*tu; vt = tfac*tv; pt = tfac*tp; wt = tfac*tw;

pdiff=pt(floor(N/2),floor(N/2))-p2(floor(N/2),floor(N/2));
uc = u2; vc = v2; pc = p2+pdiff; wc = Dx*vc-uc*Dy;  divc = Dx*uc+vc*Dy;






